#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import TestCase

from google.cloud.sqlcommenter import generate_sql_comment


class GenerateSqlCommentTests(TestCase):
    def test_no_args(self):
        self.assertEqual(generate_sql_comment(), '')

    def test_end_comment_escaping(self):
        self.assertIn("a='%%2A/abc'", generate_sql_comment(a='*/abc'))

    def test_bytes(self):
        self.assertIn("a='%%2A/abc'", generate_sql_comment(a=b'*/abc'))

    def test_url_quoting(self):
        self.assertIn("foo='bar%%2Cbaz'", generate_sql_comment(foo='bar,baz'))



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import TestCase

import psycopg2.extensions
from forbiddenfruit import curse
from google.cloud.sqlcommenter import url_quote
from google.cloud.sqlcommenter.psycopg2.extension import CommenterCursorFactory

from ..compat import mock
from ..opencensus_mock import mock_opencensus_tracer
from ..opentelemetry_mock import mock_opentelemetry_context


class Psycopg2TestCase(TestCase):

    def assertSQL(self, sql, **kwargs):
        def execute(self, sql, args=None):
            pass
        mocked_execute = mock.create_autospec(execute, return_value='worked')
        curse(psycopg2.extensions.cursor, 'execute', mocked_execute)
        cursor = CommenterCursorFactory(**kwargs)
        self.assertEqual(cursor.execute(None, 'SELECT 1;'), 'worked')
        mocked_execute.assert_called_with(None, sql, None)


class Tests(Psycopg2TestCase):

    def test_no_args(self):
        self.assertSQL('SELECT 1;')

    def test_db_driver(self):
        self.assertSQL(
            "SELECT 1; /*db_driver='psycopg2%%3A{}'*/".format(url_quote(psycopg2.__version__)),
            with_db_driver=True,
        )

    def test_dbapi_threadsafety(self):
        self.assertSQL(
            "SELECT 1; /*dbapi_threadsafety={}*/".format(psycopg2.threadsafety),
            with_dbapi_threadsafety=True,
        )

    def test_driver_paramstyle(self):
        self.assertSQL(
            "SELECT 1; /*driver_paramstyle='{}'*/".format(psycopg2.paramstyle),
            with_driver_paramstyle=True,
        )

    def test_dbapi_level(self):
        self.assertSQL(
            "SELECT 1; /*dbapi_level='{}'*/".format(url_quote(psycopg2.apilevel)),
            with_dbapi_level=True,
        )

    def test_libpq_version(self):
        self.assertSQL(
            "SELECT 1; /*libpq_version={}*/".format(url_quote(psycopg2.__libpq_version__)),
            with_libpq_version=True,
        )

    def test_opencensus(self):
        with mock_opencensus_tracer():
            self.assertSQL(
                "SELECT 1; /*traceparent='00-trace%%20id-span%%20id-00',"
                "tracestate='congo%%3Dt61rcWkgMzE%%2Crojo%%3D00f067aa0ba902b7'*/",
                with_opencensus=True,
            )

    def test_opentelemetry(self):
        with mock_opentelemetry_context():
            self.assertSQL(
                "SELECT 1; /*traceparent='00-000000000000000000000000deadbeef-000000000000beef-00',"
                "tracestate='some_key%%3Dsome_value'*/",
                with_opentelemetry=True,
            )

    def test_both_opentelemetry_and_opencensus_warn(self):
        with mock.patch(
            "google.cloud.sqlcommenter.psycopg2.extension.logger"
        ) as logger_mock, mock_opencensus_tracer(), mock_opentelemetry_context():
            self.assertSQL(
                "SELECT 1; /*traceparent='00-000000000000000000000000deadbeef-000000000000beef-00',"
                "tracestate='some_key%%3Dsome_value'*/",
                with_opentelemetry=True,
                with_opencensus=True,
            )
            self.assertEqual(len(logger_mock.warning.mock_calls), 1)


class FlaskTests(Psycopg2TestCase):
    flask_info = {
        'framework': 'flask',
        'controller': 'c',
        'route': '/',
    }

    @mock.patch('google.cloud.sqlcommenter.psycopg2.extension.get_flask_info', return_value=flask_info)
    def test_all_data(self, get_info):
        self.assertSQL(
            "SELECT 1; /*controller='c',framework='flask',route='/'*/",
        )

    @mock.patch('google.cloud.sqlcommenter.psycopg2.extension.get_flask_info', return_value=flask_info)
    def test_framework_disabled(self, get_info):
        self.assertSQL(
            "SELECT 1; /*controller='c',route='/'*/",
            with_framework=False,
        )

    @mock.patch('google.cloud.sqlcommenter.psycopg2.extension.get_flask_info', return_value=flask_info)
    def test_controller_disabled(self, get_info):
        self.assertSQL(
            "SELECT 1; /*framework='flask',route='/'*/",
            with_controller=False,
        )

    @mock.patch('google.cloud.sqlcommenter.psycopg2.extension.get_flask_info', return_value=flask_info)
    def test_route_disabled(self, get_info):
        self.assertSQL(
            "SELECT 1; /*controller='c',framework='flask'*/",
            with_route=False,
        )



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

try:
    from opencensus.trace import execution_context
    from opencensus.trace.propagation import trace_context_http_header_format
except ImportError:
    execution_context = None


def get_opencensus_values():
    """
    Return the OpenCensus Trace and Span IDs if Span ID is set in the
    OpenCensus execution context.
    """
    if execution_context:
        span_ctx = execution_context.get_opencensus_tracer().span_context
        span_id = span_ctx.span_id
        if span_id:
            # Insert the W3C TraceContext generated
            w3C_trace_ctx = trace_context_http_header_format.TraceContextPropagator()
            return w3C_trace_ctx.to_headers(span_ctx)
    else:
        raise ImportError('opencensus is not installed.')
    return {}



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import TestCase

import six
from google.cloud.sqlcommenter.opencensus import get_opencensus_values

from ..compat import mock
from ..opencensus_mock import mock_opencensus_tracer


class OpenCensusTests(TestCase):
    def test_not_installed(self):
        with mock.patch('google.cloud.sqlcommenter.opencensus.execution_context', new=None):
            with six.assertRaisesRegex(self, ImportError, 'opencensus is not installed'):
                get_opencensus_values()

    def test_no_values(self):
        self.assertEqual(get_opencensus_values(), {})

    def test_values(self):
        with mock_opencensus_tracer():
            self.assertEqual(get_opencensus_values(), {
                'traceparent': '00-trace id-span id-00',
                'tracestate': 'congo=t61rcWkgMzE,rojo=00f067aa0ba902b7',
            })



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import TestCase

import six
from google.cloud.sqlcommenter.opentelemetry import get_opentelemetry_values

from ..compat import mock, skipIfPy2
from ..opentelemetry_mock import mock_opentelemetry_context


@skipIfPy2
class OpenTelemetryTests(TestCase):
    def test_not_installed(self):
        with mock.patch("google.cloud.sqlcommenter.opentelemetry.propagator", new=None):
            with six.assertRaisesRegex(
                self, ImportError, "OpenTelemetry is not installed"
            ):
                get_opentelemetry_values()

    def test_no_values(self):
        self.assertEqual(get_opentelemetry_values(), {})

    def test_trace_parent_and_tracestate(self):
        with mock_opentelemetry_context():
            self.assertEqual(
                get_opentelemetry_values(),
                {
                    "traceparent": "00-000000000000000000000000deadbeef-000000000000beef-00",
                    "tracestate": "some_key=some_value",
                },
            )

    def test_trace_flags(self):
        from opentelemetry import trace
        with mock_opentelemetry_context(trace_flags=trace.TraceFlags.SAMPLED):
            self.assertEqual(
                get_opentelemetry_values(),
                {
                    "traceparent": "00-000000000000000000000000deadbeef-000000000000beef-01",
                    "tracestate": "some_key=some_value",
                },
            )

    def test_empty_trace_state(self):
        with mock_opentelemetry_context(trace_state={}):
            self.assertEqual(
                get_opentelemetry_values(),
                {
                    "traceparent": "00-000000000000000000000000deadbeef-000000000000beef-00",
                },
            )



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

if sys.version_info.major <= 2:
    import urllib
    url_quote_fn = urllib.quote
else:
    import urllib.parse
    url_quote_fn = urllib.parse.quote

KEY_VALUE_DELIMITER = ','


def generate_sql_comment(**meta):
    """
    Return a SQL comment with comma delimited key=value pairs created from
    **meta kwargs.
    """
    if not meta:  # No entries added.
        return ''

    # Sort the keywords to ensure that caching works and that testing is
    # deterministic. It eases visual inspection as well.
    return ' /*' + KEY_VALUE_DELIMITER.join(
        '{}={!r}'.format(url_quote(key), url_quote(value)) for key, value in sorted(meta.items())
        if value is not None
    ) + '*/'


def url_quote(s):
    if not isinstance(s, (str, bytes)):
        return s
    quoted = url_quote_fn(s)
    # Since SQL uses '%' as a keyword, '%' is a by-product of url quoting
    # e.g. foo,bar --> foo%2Cbar
    # thus in our quoting, we need to escape it too to finally give
    #      foo,bar --> foo%%2Cbar
    return quoted.replace('%', '%%')



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import django
from django.db import connection
from django.http import HttpRequest
from django.test import TestCase, override_settings, modify_settings
from django.urls import resolve, reverse
from google.cloud.sqlcommenter.django.middleware import SqlCommenter, QueryWrapper

from ..compat import mock
from ..opencensus_mock import mock_opencensus_tracer
from ..opentelemetry_mock import mock_opentelemetry_context
from . import views

# Test middleware to call execute_wrapper again
# Adding the middleware twice in modify_settings
# doesn't work. The middleware is only used once
# if used in modify_settings
class TestMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        with connection.execute_wrapper(QueryWrapper(request)):
            return self.get_response(request)

# Query log only active if DEBUG=True.
@override_settings(DEBUG=True)
class Tests(TestCase):

    @staticmethod
    def get_request(path):
        request = HttpRequest()
        request.resolver_match = resolve(path)
        return request

    def get_query(self, path='/'):
        SqlCommenter(views.home)(self.get_request(path))
        # Query with comment added by QueryWrapper and unaltered query added
        # by Django's CursorDebugWrapper.
        self.assertEqual(len(connection.queries), 2)
        return connection.queries[0]

    def assertRoute(self, route, query):
        # route available in Django 2.2 and later.
        if django.VERSION < (2, 2):
            self.assertNotIn('route=', query)
        else:
            self.assertIn("route='%s'" % route, query)

    def test_basic(self):
        query = self.get_query()
        self.assertIn("/*controller='home'", query)
        # Expecting url_quoted("framework='django:'")
        self.assertIn("framework='django%%3A" + django.get_version(), query)
        self.assertRoute('', query)

    def test_basic_disabled(self):
        with self.settings(
                SQLCOMMENTER_WITH_CONTROLLER=False, SQLCOMMENTER_WITH_ROUTE=False,
                SQLCOMMENTER_WITH_FRAMEWORK=False):
            query = self.get_query('/path/')
            self.assertNotIn('controller=', query)
            self.assertNotIn('framework=', query)
            self.assertNotIn('route=', query)

    def test_non_root_path(self):
        query = self.get_query(path='/path/')
        self.assertIn("/*controller='some-path'", query)
        self.assertRoute('path/', query)

    def test_app_path(self):
        with self.settings(SQLCOMMENTER_WITH_APP_NAME=True):
            query = self.get_query(path=reverse('app_urls:app-path'))
            self.assertIn("/*app_name='app_urls'", query)
            self.assertIn("controller='app_urls%%3Aapp-path'", query)
            self.assertRoute('app-urls/app-path/', query)

    def test_app_name_disabled(self):
        query = self.get_query(path=reverse('app_urls:app-path'))
        self.assertNotIn('app_name=', query)

    def test_empty_app_name(self):
        """An empty app_name is omitted."""
        with self.settings(SQLCOMMENTER_WITH_APP_NAME=True):
            query = self.get_query()
            self.assertNotIn("app_name=", query)

    def test_db_driver(self):
        with self.settings(SQLCOMMENTER_WITH_DB_DRIVER=True):
            query = self.get_query()
            self.assertIn("db_driver='django.db.backends.sqlite3'", query)

    def test_db_driver_disabled(self):
        query = self.get_query()
        self.assertNotIn('db_driver=', query)

    @modify_settings(MIDDLEWARE={
        'prepend': 'tests.django.tests.TestMiddleware',
        'append': 'google.cloud.sqlcommenter.django.middleware.SqlCommenter',
    })
    def test_multi_execute_wrappers(self):
        # Raises error if execute_wrappers fails
        self.client.get('/')

    def test_opencensus_disabled(self):
        """Opencensus fields are omitted by default."""
        with mock_opencensus_tracer():
            query = self.get_query()
            self.assertNotIn("traceparent", query)
            self.assertNotIn("tracestate", query)

    def test_opentelemetry_disabled(self):
        """OpenTelemetry fields are omitted by default."""
        query = self.get_query()
        self.assertNotIn("traceparent", query)
        self.assertNotIn("tracestate", query)

    def test_opencensus_enabled(self):
        with mock_opencensus_tracer(), self.settings(SQLCOMMENTER_WITH_OPENCENSUS=True):
            query = self.get_query()
            self.assertIn(
                "traceparent='00-trace%%20id-span%%20id-00',"
                "tracestate='congo%%3Dt61rcWkgMzE%%2Crojo%%3D00f067aa0ba902b7'",
                query,
            )

    def test_opentelemetry_enabled(self):
        with mock_opentelemetry_context(), self.settings(SQLCOMMENTER_WITH_OPENTELEMETRY=True):
            query = self.get_query()
            self.assertIn(
                "traceparent='00-000000000000000000000000deadbeef-000000000000beef-00',"
                "tracestate='some_key%%3Dsome_value'",
                query,
            )

    def test_both_opentelemetry_and_opencensus_warn(self):
        with mock.patch(
            "google.cloud.sqlcommenter.django.middleware.logger"
        ) as logger_mock, self.settings(
            SQLCOMMENTER_WITH_OPENCENSUS=True
        ), self.settings(
            SQLCOMMENTER_WITH_OPENTELEMETRY=True
        ):
            self.get_query()
            self.assertEqual(len(logger_mock.warning.mock_calls), 1)



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This integration implements the before_cursor_execute hook factory as per:
#   https://kite.com/python/docs/sqlalchemy.events.ConnectionEvents.before_execute
from __future__ import absolute_import

import logging

import sqlalchemy
from google.cloud.sqlcommenter import generate_sql_comment
from google.cloud.sqlcommenter.flask import get_flask_info
from google.cloud.sqlcommenter.opencensus import get_opencensus_values
from google.cloud.sqlcommenter.opentelemetry import get_opentelemetry_values

logger = logging.getLogger(__name__)


def BeforeExecuteFactory(
        with_framework=True, with_controller=True, with_route=True,
        with_opencensus=False, with_opentelemetry=False, with_db_driver=False,
        with_db_framework=False):

    attributes = {
        'framework': with_framework,
        'controller': with_controller,
        'route': with_route,
        'db_driver': with_db_driver,
        'db_framework': with_db_framework,
    }

    def before_cursor_execute(conn, cursor, sql, parameters, context, executemany):
        data = dict(
            # TODO: Figure out how to retrieve the exact driver version.
            db_driver=conn.engine.driver,
            # Driver/framework centric information.
            db_framework='sqlalchemy:%s' % sqlalchemy.__version__,
        )

        # Because sqlalchemy is a plain database connectivity module,
        # folks using it in a web framework such as flask will
        # use it in unison with flask but initialize the parts disjointly,
        # unlike Django which uses ORMs directly as part of the framework.
        data.update(get_flask_info())

        # Filter down to just the requested attributes.
        data = {k: v for k, v in data.items() if attributes.get(k)}

        if with_opencensus and with_opentelemetry:
            logger.warning(
                "with_opencensus and with_opentelemetry were enabled. "
                "Only use one to avoid unexpected behavior"
            )
        if with_opencensus:
            data.update(get_opencensus_values())
        if with_opentelemetry:
            data.update(get_opentelemetry_values())

        sql_comment = generate_sql_comment(**data)

        # TODO: Check if the database type is MySQL and figure out
        # if we should prepend comments because MySQL server truncates
        # logs greater than 1kB.
        # See:
        #  * https://github.com/basecamp/marginalia/issues/61
        #  * https://github.com/basecamp/marginalia/pull/80
        sql += sql_comment

        return sql, parameters

    return before_cursor_execute



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

try:
    from opentelemetry.trace.propagation.tracecontext import (
        TraceContextTextMapPropagator,
    )

    propagator = TraceContextTextMapPropagator()
except ImportError:
    propagator = None


def get_opentelemetry_values():
    """
    Return the OpenTelemetry Trace and Span IDs if Span ID is set in the
    OpenTelemetry execution context.
    """
    if propagator:
        # Insert the W3C TraceContext generated
        headers = {}
        propagator.inject(headers)
        return headers
    else:
        raise ImportError("OpenTelemetry is not installed.")



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import django
from django.db import connection
from django.db.backends.utils import CursorDebugWrapper
from google.cloud.sqlcommenter import generate_sql_comment
from google.cloud.sqlcommenter.opencensus import get_opencensus_values
from google.cloud.sqlcommenter.opentelemetry import get_opentelemetry_values

django_version = django.get_version()
logger = logging.getLogger(__name__)


class SqlCommenter:
    """
    Middleware to append a comment to each database query with details about
    the framework and the execution context.
    """
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        with connection.execute_wrapper(QueryWrapper(request)):
            return self.get_response(request)


class QueryWrapper:
    def __init__(self, request):
        self.request = request

    def __call__(self, execute, sql, params, many, context):
        with_framework = getattr(django.conf.settings, 'SQLCOMMENTER_WITH_FRAMEWORK', True)
        with_controller = getattr(django.conf.settings, 'SQLCOMMENTER_WITH_CONTROLLER', True)
        with_route = getattr(django.conf.settings, 'SQLCOMMENTER_WITH_ROUTE', True)
        with_app_name = getattr(django.conf.settings, 'SQLCOMMENTER_WITH_APP_NAME', False)
        with_opencensus = getattr(django.conf.settings, 'SQLCOMMENTER_WITH_OPENCENSUS', False)
        with_opentelemetry = getattr(django.conf.settings, 'SQLCOMMENTER_WITH_OPENTELEMETRY', False)
        with_db_driver = getattr(django.conf.settings, 'SQLCOMMENTER_WITH_DB_DRIVER', False)

        if with_opencensus and with_opentelemetry:
            logger.warning(
                "SQLCOMMENTER_WITH_OPENCENSUS and SQLCOMMENTER_WITH_OPENTELEMETRY were enabled. "
                "Only use one to avoid unexpected behavior"
            )

        db_driver = context['connection'].settings_dict.get('ENGINE', '')
        resolver_match = self.request.resolver_match

        sql_comment = generate_sql_comment(
            # Information about the controller.
            controller=resolver_match.view_name if resolver_match and with_controller else None,
            # route is the pattern that matched a request with a controller i.e. the regex
            # See https://docs.djangoproject.com/en/stable/ref/urlresolvers/#django.urls.ResolverMatch.route
            # getattr() because the attribute doesn't exist in Django < 2.2.
            route=getattr(resolver_match, 'route', None) if resolver_match and with_route else None,
            # app_name is the application namespace for the URL pattern that matches the URL.
            # See https://docs.djangoproject.com/en/stable/ref/urlresolvers/#django.urls.ResolverMatch.app_name
            app_name=(resolver_match.app_name or None) if resolver_match and with_app_name else None,
            # Framework centric information.
            framework=('django:%s' % django_version) if with_framework else None,
            # Information about the database and driver.
            db_driver=db_driver if with_db_driver else None,
            **get_opencensus_values() if with_opencensus else {},
            **get_opentelemetry_values() if with_opentelemetry else {}
        )

        # TODO: MySQL truncates logs > 1024B so prepend comments
        # instead of statements, if the engine is MySQL.
        # See:
        #  * https://github.com/basecamp/marginalia/issues/61
        #  * https://github.com/basecamp/marginalia/pull/80
        sql += sql_comment

        # Add the query to the query log if debugging.
        if context['cursor'].__class__ is CursorDebugWrapper:
            context['connection'].queries_log.append(sql)

        return execute(sql, params, many, context)



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

import flask
import pytest
from google.cloud.sqlcommenter.flask import get_flask_info

from .app import app

# Following the example provided at
# https://flask.palletsprojects.com/en/1.1.x/testing/#the-testing-skeleton


@pytest.fixture
def client():
    app.config['TESTING'] = True
    with app.test_client() as client:
        yield client


def test_get_flask_info_in_request_context(client):
    expected = {
        'controller': 'flask_info',
        'framework': 'flask:%s' % flask.__version__,
        'route': '/flask-info',
    }
    resp = client.get('/flask-info')
    assert json.loads(resp.data.decode('utf-8')) == expected


def test_get_flask_info_in_404_error_context(client):
    expected = {
        'framework': 'flask:%s' % flask.__version__,
    }
    resp = client.get('/doesnt-exist')
    assert json.loads(resp.data.decode('utf-8')) == expected


def test_get_flask_info_outside_request_context(client):
    assert get_flask_info() == {}



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from setuptools import find_packages, setup


def read_file(filename):
    with open(os.path.join(os.path.dirname(__file__), filename)) as file:
        return file.read()


setup(
    name='google-cloud-sqlcommenter',
    version='2.0.0',
    author='Google Developers',
    author_email='sqlcommenter@googlegroups.com',
    description=('Augment SQL statements with meta information about frameworks and the running environment.'),
    long_description=read_file('README.md'),
    long_description_content_type='text/markdown',
    license='BSD',
    packages=find_packages(exclude=['tests']),
    extras_require={
        'django': ['django >= 1.11'],
        'flask': ['flask'],
        'psycopg2': ['psycopg2'],
        'sqlalchemy': ['sqlalchemy'],
        'opencensus': ['opencensus'],
        'opentelemetry': ["opentelemetry-api ~= 1.0"],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Utilities',
        'Framework :: Django',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
    ],
)



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

try:
    import flask
    from flask import request
except ImportError:
    flask = None


def get_flask_info():
    """
    Get available info from the current flask request, if we're in a
    flask request-response cycle. Else, return an empty dict.
    """
    info = {}
    # https://flask.palletsprojects.com/en/1.1.x/api/#flask.has_request_context
    if flask and request:
        info['framework'] = 'flask:%s' % flask.__version__
        if request.endpoint:
            info['controller'] = request.endpoint
        if request.url_rule and request.url_rule.rule:
            info['route'] = request.url_rule.rule
    return info



#!/usr/bin/python
#
# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

import psycopg2
import psycopg2.extensions
from google.cloud.sqlcommenter import generate_sql_comment
from google.cloud.sqlcommenter.flask import get_flask_info
from google.cloud.sqlcommenter.opencensus import get_opencensus_values
from google.cloud.sqlcommenter.opentelemetry import get_opentelemetry_values

logger = logging.getLogger(__name__)


# This integration extends psycopg2.extensions.cursor
# by implementing a custom execute method.
#
# By default, it doesn't enable adding trace_id and span_id
# to SQL comments due to their ephemeral nature. You can opt-in
# by instead setting with_opencensus=True
def CommenterCursorFactory(
        with_framework=True, with_controller=True, with_route=True,
        with_opencensus=False, with_opentelemetry=False, with_db_driver=False,
        with_dbapi_threadsafety=False, with_dbapi_level=False,
        with_libpq_version=False, with_driver_paramstyle=False):

    attributes = {
        'framework': with_framework,
        'controller': with_controller,
        'route': with_route,
        'db_driver': with_db_driver,
        'dbapi_threadsafety': with_dbapi_threadsafety,
        'dbapi_level': with_dbapi_level,
        'libpq_version': with_libpq_version,
        'driver_paramstyle': with_driver_paramstyle,
    }

    class CommenterCursor(psycopg2.extensions.cursor):

        def execute(self, sql, args=None):
            data = dict(
                # Psycopg2/framework information
                db_driver='psycopg2:%s' % psycopg2.__version__,
                dbapi_threadsafety=psycopg2.threadsafety,
                dbapi_level=psycopg2.apilevel,
                libpq_version=psycopg2.__libpq_version__,
                driver_paramstyle=psycopg2.paramstyle,
            )

            # Because psycopg2 is a plain database connectivity module,
            # folks using it in a web framework such as flask will
            # use it in unison with flask but initialize the parts disjointly,
            # unlike Django which uses ORMs directly as part of the framework.
            data.update(get_flask_info())

            # Filter down to just the requested attributes.
            data = {k: v for k, v in data.items() if attributes.get(k)}

            if with_opencensus and with_opentelemetry:
                logger.warning(
                    "with_opencensus and with_opentelemetry were enabled. "
                    "Only use one to avoid unexpected behavior"
                )
            if with_opencensus:
                data.update(get_opencensus_values())
            if with_opentelemetry:
                data.update(get_opentelemetry_values())

            sql += generate_sql_comment(**data)

            return psycopg2.extensions.cursor.execute(self, sql, args)

    return CommenterCursor
